@@ -406,10 +406,10 @@ int lucas_lehmer(UV p)
     if ( (q%8==1 || q%8==7) &&                 /* factor must be 1 or 7 mod 8 */
          q % 3 && q % 5 && q % 7 && q % 11 && q % 13) {  /* and must be prime */
       if (1 && q < (1ULL << (BITS_PER_WORD/2)) ) {
-        UV b = 1, k = pbits;
-        while (k--) {
+        UV b = 1, j = pbits;
+        while (j--) {
           b = (b*b) % q;
-          if (p & (UVCONST(1) << k)) { b *= 2; if (b >= q) b -= q; }
+          if (p & (UVCONST(1) << j)) { b *= 2; if (b >= q) b -= q; }
         }
         if (b == 1)
           { mpz_clear(mp); mpz_clear(t); return 0; }
@@ -965,9 +965,9 @@ int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
     mpz_set_ui(wm, 2);
     mpz_set(wm1, w1);
     {
-      UV b = mpz_sizeinbase(m, 2);
-      while (b-- > 0) {
-        if (mpz_tstbit(m, b)) {
+      UV bit = mpz_sizeinbase(m, 2);
+      while (bit-- > 0) {
+        if (mpz_tstbit(m, bit)) {
           mpz_mul(t, wm, wm1);
           mpz_sub(wm, t, w1);
           mpz_mul(t, wm1, wm1);
@@ -1134,9 +1134,10 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
   if ( (mpz_cmp_ui(ra,1) == 0) && (mpz_cmp(rb, d) == 0) )
     rval = 1;
 
+  mpz_clear(d);
   mpz_clear(ta); mpz_clear(tb);
+  mpz_clear(a);  mpz_clear(b);
   mpz_clear(ra); mpz_clear(rb);
-  mpz_clear(a); mpz_clear(b);
   mpz_clear(t);
   return rval;
 }
@@ -1705,16 +1706,16 @@ int _GMP_is_aks_prime(mpz_t n)
     slim = (UV) (2*t*(r-1));
     c2 = dlogn * floor(sqrt(r));
     { /* Binary search for first s in [1,slim] where x >= 0 */
-      UV i = 1;
-      UV j = slim;
-      while (i < j) {
-        s = i + (j-i)/2;
+      UV bi = 1;
+      UV bj = slim;
+      while (bi < bj) {
+        s = bi + (bj-bi)/2;
         mpz_bin_uiui(tmp, r+s-1, s);
         x = mpz_logn(tmp) / c2 - 1.0;
-        if (x < 0)  i = s+1;
-        else        j = s;
+        if (x < 0)  bi = s+1;
+        else        bj = s;
       }
-      s = i-1;
+      s = bi-1;
     }
     s = (s+3) >> 1;
     /* Bornemann checks factors up to (s-1)^2, we check to max(r,s) */
@@ -1958,15 +1959,18 @@ void _GMP_pn_primorial(mpz_t prim, UV n)
   }
   prime_iterator_destroy(&iter);
 }
-void _GMP_primorial(mpz_t prim, mpz_t n)
+void _GMP_primorial(mpz_t prim, UV n)
 {
+#if (__GNU_MP_VERSION > 5) || (__GNU_MP_VERSION == 5 && __GNU_MP_VERSION_MINOR >= 1)
+  mpz_primorial_ui(prim, n);
+#else
   UV p = 2;
   PRIME_ITERATOR(iter);
 
-  if (mpz_cmp_ui(n, 1000) < 0) {
+  if (n < 1000) {
     /* Simple linear multiplication, one at a time */
     mpz_set_ui(prim, 1);
-    while (mpz_cmp_ui(n, p) >= 0) {
+    while (p <= n) {
       mpz_mul_ui(prim, prim, p);
       p = prime_iterator_next(&iter);
     }
@@ -1976,7 +1980,7 @@ void _GMP_primorial(mpz_t prim, mpz_t n)
     UV i;
     for (i = 0; i < 16; i++)  mpz_init_set_ui(t[i], 1);
     i = 0;
-    while (mpz_cmp_ui(n, p) >= 0) {
+    while (p <= n) {
       mpz_mul_ui(t[i&15], t[i&15], p);
       p = prime_iterator_next(&iter);
       i++;
@@ -1986,6 +1990,7 @@ void _GMP_primorial(mpz_t prim, mpz_t n)
     for (i = 0; i < 16; i++)  mpz_clear(t[i]);
   }
   prime_iterator_destroy(&iter);
+#endif
 }
 
 /* Luschny's version of the "Brent-Harvey" method */
@@ -2079,7 +2084,7 @@ char* harmreal(mpz_t zn, unsigned long prec) {
   mpf_set_z(fnum, num);  mpf_set_z(fden, den);
   mpz_clear(den); mpz_clear(num);
 
-  mpf_init2(res, (mp_bitcnt_t) (8+prec*3.4) );
+  mpf_init2(res, (unsigned long) (8+prec*3.4) );
   mpf_div(res, fnum, fden);
   mpf_clear(fnum);  mpf_clear(fden);
 
@@ -2175,10 +2180,10 @@ void binomial(mpz_t r, UV n, UV k)
       if (n % prime < k % prime)
         PUSHP(prime);
     } else {
-      UV N = n, K = k, p = 1, r = 0;
+      UV N = n, K = k, p = 1, s = 0;
       while (N > 0) {
-        r = (N % prime) < (K % prime + r) ? 1 : 0;
-        if (r == 1)  p *= prime;
+        s = (N % prime) < (K % prime + s) ? 1 : 0;
+        if (s == 1)  p *= prime;
         N /= prime;
         K /= prime;
       }
@@ -2995,8 +3000,9 @@ void exp_mangoldt(mpz_t res, mpz_t n)
 
 uint32_t* partial_sieve(mpz_t start, UV length, UV maxprime)
 {
-  UV p, m, pos;
+  uint32_t const premark[3] = { 0x92492492, 0x24924924, 0x49249249 };
   uint32_t* comp;
+  UV p, m, pos;
   mpz_t t;
   PRIME_ITERATOR(iter);
 
@@ -3007,14 +3013,19 @@ uint32_t* partial_sieve(mpz_t start, UV length, UV maxprime)
   MPUassert(length > 0, "partial sieve given zero length");
   mpz_sub_ui(start, start, 1);
   if (length & 1) length++;
-  Newz(0, comp, (length+63)/64, uint32_t);
   mpz_init(t);
 
+  /* Allocate odds-only array, and pre-mark with 3s */
+  New(0, comp, (length+63)/64, uint32_t);
+  pos = (3 - (mpz_fdiv_ui(start, 6) >> 1)) % 3;
+  for (m = 0; m < (length+63)/64; m++)
+    comp[m] = premark[ (pos++) % 3 ];
+
   p = prime_iterator_next(&iter);
-  for (p = 3; p <= maxprime; p = prime_iterator_next(&iter)) {
-    m = mpz_mod_ui(t, start, p);
-    pos = (m == 0) ? 0 : p-m;    /* First multiple of p after start  */
-    if (!(pos & 1)) pos += p;    /* Make sure it is odd.             */
+  p = prime_iterator_next(&iter);
+  for (p = 5; p <= maxprime; p = prime_iterator_next(&iter)) {
+    pos = p - mpz_fdiv_ui(start,p);  /* First multiple of p after start  */
+    if (!(pos & 1)) pos += p;        /* Make sure it is odd.             */
     while (pos < length) {
       SETAVAL(comp, pos);
       pos += 2*p;
