@@ -19,7 +19,12 @@ void set_verbose_level(int level) { _verbose = level; }
 static gmp_randstate_t _randstate;
 gmp_randstate_t* get_randstate(void) { return &_randstate; }
 void init_randstate(unsigned long seed) {
+#if (__GNU_MP_VERSION > 4) || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
+  /* MT was added in GMP 4.2 released in 2006. */
   gmp_randinit_mt(_randstate);
+#else
+  gmp_randinit_default(_randstate);
+#endif
   gmp_randseed_ui(_randstate, seed);
 }
 void clear_randstate(void) {  gmp_randclear(_randstate);  }
@@ -356,9 +361,10 @@ void poly_mod_sqr(mpz_t* px, mpz_t* ptmp, UV r, mpz_t mod)
 }
 #endif
 
-#if 0
+#if (__GNU_MP_VERSION < 4) || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR < 1)
 /* Binary segmentation, using simple shift+add method for processing p.
  * Faster than twiddling bits, but not nearly as fast as import/export.
+ * mpz_import and mpz_export were added in GMP 4.1 released in 2002.
  */
 void poly_mod_mul(mpz_t* px, mpz_t* py, UV r, mpz_t mod, mpz_t p, mpz_t p2, mpz_t t)
 {
@@ -397,7 +403,8 @@ void poly_mod_mul(mpz_t* px, mpz_t* py, UV r, mpz_t mod, mpz_t p, mpz_t p2, mpz_
   for (i = 0; i < r; i++)
     mpz_mod(px[i], px[i], mod);
 }
-#endif
+
+#else
 
 /* Binary segmentation, using import/export method for processing p.
  * Thanks to Dan Bernstein's 2007 Quartic paper.
@@ -448,6 +455,7 @@ void poly_mod_mul(mpz_t* px, mpz_t* py, UV r, mpz_t mod, mpz_t p, mpz_t p2, mpz_
     Safefree(s);
   }
 }
+#endif
 
 void poly_mod_pow(mpz_t *pres, mpz_t *pn, mpz_t power, UV r, mpz_t mod)
 {
@@ -963,6 +971,23 @@ void polyz_roots_modp(mpz_t** roots, long *nroots, long maxroots,
   if (dP == 0)
     return;
 
+  /* Do degree 1 or 2 explicitly */
+  if (dP == 1) {
+    New(0, *roots, 1, mpz_t);
+    mpz_init((*roots)[0]);
+    polyz_root_deg1( (*roots)[0], pP, NMOD );
+    *nroots = 1;
+    return;
+  }
+  if (dP == 2) {
+    New(0, *roots, 2, mpz_t);
+    mpz_init((*roots)[0]);
+    mpz_init((*roots)[1]);
+    polyz_root_deg2( (*roots)[0], (*roots)[1], pP, NMOD );
+    *nroots = 2;
+    return;
+  }
+
   /* Allocate space for the maximum number of roots (plus 1 for safety) */
   New(0, *roots, dP+1, mpz_t);
   for (i = 0; i <= dP; i++)
@@ -971,18 +996,6 @@ void polyz_roots_modp(mpz_t** roots, long *nroots, long maxroots,
   if (maxroots > dP || maxroots == 0)
     maxroots = dP;
 
-  /* Do degree 1 or 2 explicitly */
-  if (dP == 1) {
-    polyz_root_deg1( (*roots)[0], pP, NMOD );
-    *nroots = 1;
-    return;
-  }
-  if (dP == 2) {
-    polyz_root_deg2( (*roots)[0], (*roots)[1], pP, NMOD );
-    *nroots = 2;
-    return;
-  }
-
   polyz_roots(*roots, nroots, maxroots, pP, dP, NMOD, p_randstate);
   /* This could be just really bad luck.  Let the caller handle it. */
   /* if (*nroots == 0) croak("failed to find roots\n"); */
